'use strict';

var collection = require('@discordjs/collection');
var Store_cjs = require('./Store.cjs');

var __defProp = Object.defineProperty;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __name = (target, value) => __defProp(target, "name", { value, configurable: true });
var __publicField = (obj, key, value) => {
  __defNormalProp(obj, typeof key !== "symbol" ? key + "" : key, value);
  return value;
};
var _AliasStore = class _AliasStore extends Store_cjs.Store {
  constructor() {
    super(...arguments);
    /**
     * The aliases referencing to pieces.
     */
    __publicField(this, "aliases", new collection.Collection());
  }
  /**
   * Looks up the name by the store, falling back to an alias lookup.
   * @param key The key to look for.
   */
  get(key) {
    return super.get(key) ?? this.aliases.get(key);
  }
  /**
   * Checks whether a key is in the store, or is an alias
   * @param key The key to check
   */
  has(key) {
    return super.has(key) || this.aliases.has(key);
  }
  /**
   * Unloads a piece given its instance or its name, and removes all the aliases.
   * @param name The name of the file to load.
   * @return Returns the piece that was unloaded.
   */
  unload(name) {
    const piece = this.resolve(name);
    for (const alias of piece.aliases) {
      const aliasPiece = this.aliases.get(alias);
      if (aliasPiece === piece)
        this.aliases.delete(alias);
    }
    return super.unload(piece);
  }
  /**
   * Inserts a piece into the store, and adds all the aliases.
   * @param piece The piece to be inserted into the store.
   * @return The inserted piece.
   */
  async insert(piece) {
    for (const key of piece.aliases) {
      this.aliases.set(key, piece);
    }
    return super.insert(piece);
  }
};
__name(_AliasStore, "AliasStore");
var AliasStore = _AliasStore;

exports.AliasStore = AliasStore;
//# sourceMappingURL=out.js.map
//# sourceMappingURL=AliasStore.cjs.map